--    Copyright (C) 2004 by neuron                                          
--    aagaande@gmail.com                                             
--                                                                          
--    This program is free software; you can redistribute it and--or modify  
--    it under the terms of the GNU General Public License as published by  
--    the Free Software Foundation; either version 2 of the License, or     
--    (at your option) any later version.                                   
--                                                                          
--    This program is distributed in the hope that it will be useful,       
--    but WITHOUT ANY WARRANTY; without even the implied warranty of        
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
--    GNU General Public License for more details.                          
--                                                                          
--    You should have received a copy of the GNU General Public License     
--    along with this program; if not, write to the                         
--    Free Software Foundation, Inc.,                                       
--    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

local moduleName = "Coloring"
local Coloring = nQuestLog:NewModule(moduleName,"AceEvent-2.0","AceHook-2.1", "AceConsole-2.0")
local crayon = AceLibrary("Crayon-2.0")
local tourist = AceLibrary("Tourist-2.0")

local L = AceLibrary("AceLocale-2.2"):new(moduleName)
L:RegisterTranslations("enUS", function() return {
    ["Description"] = "Deals with coloring everything properly",
    
    ["Color by level"] = true,
	["Color by progress"] = true,
	["Change color settings for"] = true,
    
    ["trackerHeader"] = "Minion header color",
	["trackerBorderColor"] = "Minion border color",
	["trackerBackgroundColor"] = "Minion background color",
				
	["zoneHeader"] = "Zone header",
	["zoneLevel"] = "Zone level",
		
	["questHeader"] = "Quest header",
	["questHighlight"] = "Quest highlight",
	["questProgress"] = "Quest progress",
	["questFailed"] = "Quest Failed text",
	["questGoto"] = "Quest Goto text",
	["questDone"] = "Quest Done text",
		
	["objectiveHeader"] = "Objective header",
	["objectiveProgress"] = "Objective progress",
    
} end)

function Coloring:OnModuleLoad()
	-- We'd want to set self.name and self.description here? (used for the menu made by parent atleast)
	self.description = L["Description"]
	
	self.core:RegisterDefaults(moduleName, "profile", {
		["enabled"] = true,
	
		["trackerHeader"] = {r=226,g=142,b=59},
		["trackerBorderColor"] = {r=255,g=255,b=255,a=1},
		["trackerBackgroundColor"] = {r=0,g=0,b=0,a=0.6},
				
		["zoneHeader"] = {r=119,g=187,b=255,threshold=false},
		["zoneLevel"] = {r=119,g=187,b=255,threshold=false},
		
		["questHeader"] = {r=255,g=255,b=255,threshold=true},
		["questHighlight"] = {r=229,g=153,b=153,a=0.8},
		["questProgress"] = {r=255,g=255,b=255,threshold=true},
		["questFailed"] = {r=255,g=255,b=255},
		["questGoto"] = {r=255,g=255,b=255},
		["questDone"] = {r=255,g=255,b=255},
		
		["objectiveHeader"] = {r=255,g=255,b=255,threshold=false},
		["objectiveProgress"] = {r=255,g=255,b=255,threshold=true},
	})
end

function Coloring:GetMenu(oldMenu)
	local menu = {
		type="group",
		name=moduleName,
		order=oldMenu.order,
		desc=self.description,
		args = {
			toggleEnabled = oldMenu,
		}
	}
	
	local colorList = {
		{ name="trackerHeader" },
		{ name="trackerBorderColor" },
		{ name="trackerBackgroundColor" },
		
		{ name="zoneHeader",extra="Color by level" },
		{ name="zoneLevel",extra="Color by level" },
		
		{ name="questHeader", extra="Color by progress" },
		{ name="questHighlight" },
		{ name="questProgress", extra="Color by progress" },
		{ name="questFailed"},
		{ name="questGoto"},
		{ name="questDone"},
		
		
		{ name="objectiveHeader", extra="Color by progress"},
		{ name="objectiveProgress", extra="Color by progress"},
	}
	
	local function getColor(var)
		return self.db.profile[var].r / 255, self.db.profile[var].g / 255, self.db.profile[var].b / 255, self.db.profile[var].a
	end
	
	local function setColor(var,r,g,b,a)
		self.db.profile[var] = {r=r * 255,g=g * 255,b=b * 255,a=a}
		self.core:RefreshTracker()
	end
	
	local function getToggle(var)
		return self.db.profile[var].threshold
	end
	
	local function setToggle(var,v)
		self.db.profile[var].threshold = v
		self.core:RefreshTracker()
	end
	
	
	-- Todo make disable thing in menu's, for extra stuff and threshold?
	for i,color in ipairs(colorList) do
		local name = colorList[i].name
		local extra = colorList[i].extra
		local hasAlpha = false
		if self.db.profile[name].a then
			hasAlpha = true
		end
		
		local colorMenu = {
				type="color",
				name=L[name],
				desc=L["Change color settings for"] .. " " .. L[name],
				order = 1200 + i,
				get=function()
					return getColor(name)
				end,
				set=function(r,g,b,a)
					setColor(name,r,g,b,a)
				end,
				hasAlpha=hasAlpha
			}
		
		
		if (extra) then
			menu.args[name] = {
				type="group",
				name=L[name],
				desc=L["Change color settings for"] .. " " .. L[name],
				order = 1200 + i,
				args = {
					name = colorMenu,
					extra = {
						type="toggle",
						name=" - " .. L[extra],
						desc=L["Change color settings for"] .. " " .. L[extra],
						order=1200 + i + 1,
						get=function()
							return getToggle(name)
						end,
						set=function(v)
							setToggle(name,v)
						end,
					}
				}
			}
		else
			menu.args[name] = colorMenu
		end
	end
	
	return menu
end

function Coloring:Colorize(text,variable,range,zone,level)
	-- Colorize text, using variable for settings
	-- 	Range is for threshold colorizing, so is zone
	--		If level and threshold then colorize by level


	-- Range and zone is optional, will be used if threshold=true
	if range and self.db.profile[variable].threshold then
		-- Color by range, for example got/needed
		local color = crayon:GetThresholdHexColor(range)
		return crayon:Colorize(color,text)
	elseif zone and self.db.profile[variable].threshold then
		-- Color by zone
		local low,high = tourist:GetLevel(zone)
		local color = crayon:GetThresholdHexColor(low / high)
		return crayon:Colorize(color,text)
	elseif level and self.db.profile[variable].threshold then
		local color = GetDifficultyColor(level)
		color = string.format("%02x%02x%02x", color.r*255, color.g*255, color.b*255)
		return crayon:Colorize(color,text)
	end
	-- Take predefined color
	local color = string.format("%02x%02x%02x", self.db.profile[variable].r, self.db.profile[variable].g, self.db.profile[variable].b)
	return crayon:Colorize(color,text)
end

function Coloring:GetColor(variable)
	-- Todo, does this need optional ,range/zone variable?
	-- 	If it does make seperate function for getting range/zone colors
	--{ name="Background", type="color", desc="Set background color and transparency", variable="trackerBackgroundColor", value={r=0,g=0,b=0,a=0.6}, hasAlpha=true },
	return self.db.profile[variable]
end
